#include <stdio.h>
#include <math.h>
#include <complex.h>
#include <stdlib.h>

#define PI 3.1415926535897932384626

void dft(complex *, complex *, int);
void idft(complex *, complex *, int);
void fft(complex *, complex *, int);
void ifft(complex *, complex *, int);

double supnorm(complex *x, int N){
	/* Calculates the supremum norm */
	int k;
  double sup= 0;
  for (k= 0; k<N; k++) 
    if(cabs(x[k])>sup) sup=cabs(x[k]);
  return sup;
}

double infnorm(complex *m[2][2], int N){
	/* Calculates the sub-infinity norm */
  double err00, err01, err10, err11, err= 0;
  err00=supnorm(m[0][0], N);
  err01=supnorm(m[0][1], N);
  err10=supnorm(m[1][0], N);
  err11=supnorm(m[1][1], N);

  err= err00;
  if (err01>err) err= err01;
  if (err10>err) err= err10;
  if (err11>err) err= err11;
  return err;
}

double l1norm(complex *x, int N){
	/* Calculates L1 norm */
  int k;
  double l1= 0.;
  for(k=N/2; k<N; k++){
  	l1+= cabs(x[k]);
    l1+= cabs(x[N-1-k]);
  }
  return l1;
}

double l1tail(complex *x, int N){
	/* Calculates the norm of the tails of a Fourier array */
  int k;
  double l1= 0.;
  for (k= N/2; k<3*N/4; k++){ 
    l1+= cabs(x[k]);
    l1+= cabs(x[N-1-k]);
  }
  return l1;
}

double linftail(complex *x, int N){
	/* Calculates the sub-infinity norms of the tails */
  int k;
  double li= 0., c;
  for (k= N/2; k<3*N/4; k++){ 
    c= cabs(x[k]);
    li= li<c ? c : li;
  }
  return li;
}

void sum(complex *x, complex *y, complex *s, int N){
  int k;
  for(k=0; k<N; k++)
    s[k]= x[k]+y[k];
}

void mult(complex *x, complex *y, complex *m, int N){
  int k;
  for(k=0; k<N; k++)
    m[k]=x[k]*y[k];
}

void rest(complex *x, complex *y, complex *r, int N){
  int k;
  for(k=0; k<N; k++)
    r[k]=x[k]-y[k];
}

void esc(complex *x, complex *e, complex a, int N){
  int k;
  for(k=0; k<N; k++)
    e[k]=a*x[k];
}

void F(complex *K0, complex *K1, complex *FK0, complex *FK1, double b, double e,  int N){ 
	/* Calculates the image through the Standard Map */
  int j;
  for(j=0; j<N; j++){
    FK1[j]=K1[j]-(b/(2*PI))*csin(2*PI*K0[j])- e*csin((2*PI*j)/N);
    FK0[j]=K0[j]+FK1[j];
  }
}

void fourierrot(complex *x, complex *xrot, double om, int N){
	/* Rotates the Fourier coefficients */
  int k;
  for(k=0; k<N/2; k++)
    xrot[k]=x[k]*cexp(2*PI*I*k*om);
  for(k=N/2+1; k<N; k++)
   	xrot[k]=x[k]*cexp(2*PI*I*(k-N)*om);  
}

void cohom1(complex *y, complex *x, double lam, double mu, double om, int N){
	/* Solves the cohomological equation for lambda != mu */
  int k;
  for(k=0; k<N/2; k++)
    x[k]=y[k]/(lam-mu*cexp(2*PI*I*k*om));
  for(k=N/2+1; k<N; k++)
    x[k]=y[k]/(lam-mu*cexp(2*PI*I*(k-N)*om));
  x[N/2]=0;
}

void cohom2(complex *y, complex *x, double lam, double om, int N){
	/* Solves the cohomological equation when lambda = mu */
  int k; 
  x[0]=0;
  for(k=1; k<N/2; k++)
    x[k]=y[k]/(lam*(1-cexp(2*PI*I*k*om)));
  x[N/2]=0;
  for(k=N/2+1; k<N; k++)
    x[k]=y[k]/(lam*(1-cexp(2*PI*I*(k-N)*om)));
}

void checkcoh1(complex *x, complex *y, double lam, double mu, double om, int N){
	/* Checks the solution of the cohomological equation for lambda != mu */
  int k;
  complex *xrot=(complex*)malloc(N*sizeof(complex));
  esc(x, x, lam, N);
  fourierrot(xrot, xrot, om, N);
  esc(xrot, xrot, mu, N);
  rest(x, x, xrot, N);
  printf("\nCohom eq \t\t Eta");
  for(k=0; k<N; k++){
    printf("\n%lf+(%lf)i \t\t %lf+(%lf)i",
	  creal(x[k]), cimag(x[k]), creal(y[k]), cimag(y[k]));
  }
  free(xrot);
}

void allocm(complex *m[2][2], unsigned N){
  int i, j;
  for(i=0; i<2; i++)
    for(j=0; j<2; j++)
      m[i][j]= (complex *) malloc(N*sizeof(complex));
}

void allocv(complex *v[2], unsigned N){
  int i;
  for(i=0; i<2; i++)
    v[i]= (complex *) malloc(N*sizeof(complex));
}

void reallocm(complex *m[2][2], unsigned N){
  int i, j;
  for(i=0; i<2; i++)
    for(j=0; j<2; j++)
      m[i][j]= (complex *) realloc(m[i][j], N*sizeof(complex));	       	
}
 
void reallocv(complex *v[2], unsigned N){
  int i;
  for(i=0; i<2; i++)
   	v[i]= (complex *) realloc(v[i], N*sizeof(complex));
}

void freev(complex *v[2]){
  int i;
  for(i=0; i<2; i++)
   	free(v[i]);
}

void freem(complex *m[2][2]){
  int i, j;
  for(i=0; i<2; i++)
    for(j=0; j<2; j++)
      free(m[i][j]);	
}
 
void matrixmult(complex *x[2][2], complex *y[2][2], complex *z[2][2], int N){
  int i, j, k, l;
  complex p[2][2];
  for(k=0; k<N; k++){
    for(i=0; i<2; i++){
      for(j=0; j<2; j++){
				p[i][j]=0;
				for(l=0; l<2; l++){
	  			p[i][j]+=x[i][l][k]*y[l][j][k];
				}
      }
    }
    for(i= 0; i<2; i++)
      for(j= 0; j<2; j++)
				z[i][j][k]= p[i][j];   
  }
}

void matrixvectmult(complex *x[2][2], complex *v[2], complex *r[2], int N){
  int i, j, k;
  complex p[2];
  for(k=0; k<N; k++){
    for(i=0; i<2; i++){
      p[i]=0;
      for(j=0; j<2; j++){
				p[i]+=x[i][j][k]*v[j][k];
      }
    }
    for(i=0; i<2; i++){
      r[i][k]=p[i];
    }
  }
}

void matrixesc(complex *x[2][2], complex *y[2][2], double a, int N){
  int i, j, k;
  for(k=0; k<N; k++){
    for(i=0; i<2; i++){
      for(j=0; j<2; j++){
				y[i][j][k]=x[i][j][k]*a;
      }
    }
  }
}

void inverse(complex *a[2][2], complex *inv[2][2], int N){
	/* Inverts a matrix */
  int k;
  complex det, adj[2][2];
  for(k=0; k<N; k++){
    det=a[0][0][k]*a[1][1][k]-a[0][1][k]*a[1][0][k];
    adj[0][0]= a[1][1][k];
    adj[1][1]= a[0][0][k];
    adj[0][1]=-a[1][0][k];
    adj[1][0]=-a[0][1][k];
    inv[0][0][k]= adj[0][0]/det;
    inv[1][1][k]= adj[1][1]/det;
    inv[1][0][k]= adj[0][1]/det;
    inv[0][1][k]= adj[1][0]/det;
  }
}

void reducerr(complex *prinv[2][2], complex *dif[2][2], complex *p[2][2], complex l0, complex l1, complex *err[2][2], int N){
	/* Calculates the reducibility error */
  int k;
  matrixmult(prinv, dif, err, N);
  matrixmult(err, p, err, N);
  for(k=0; k<N; k++){
   	err[0][0][k]-=l0;
   	err[1][1][k]-=l1;
  }
}

void matrixgf(complex *grid[2][2], complex *coef[2][2], int N){
	/* Transforms an array of matrices evaluated over a grid into matrices of Fourier coefficients */
  int k;
  fft(grid[0][0], coef[0][0], N);
  fft(grid[0][1], coef[0][1], N);
  fft(grid[1][0], coef[1][0], N);
  fft(grid[1][1], coef[1][1], N);
}

void matrixfg(complex *coef[2][2], complex *grid[2][2], int N){
	/* Transforms an array of matrices of Fourier coefficients into matrices evaluated over a grid */
  int k;
  ifft(coef[0][0], grid[0][0], N);
  ifft(coef[0][1], grid[0][1], N);
  ifft(coef[1][0], grid[1][0], N);
  ifft(coef[1][1], grid[1][1], N);
}

void matrixsum(complex *x[2][2], complex *y[2][2], int N){
  int k;
  for(k=0; k<N; k++){
   	x[0][0][k]+=y[0][0][k];
    x[0][1][k]+=y[0][1][k];
    x[1][0][k]+=y[1][0][k];
    x[1][1][k]+=y[1][1][k];
  }
}

void matrixres(complex *x[2][2], complex *y[2][2], int N){
  int k;
  for(k=0; k<N; k++){
    x[0][0][k]-=y[0][0][k];
    x[0][1][k]-=y[0][1][k];
    x[1][0][k]-=y[1][0][k];
    x[1][1][k]-=y[1][1][k];
  }
}

double inverr(complex *K[2], complex *err[2], double b, double e, double om, int N){
  /* Calculates the invariance error */
  int i, k;
  complex *KrotG[2], *KrotF[2], *FK[2];
 	double error;
  allocv(KrotG, N);
  allocv(KrotF, N);
  allocv(FK, N);
  F(K[0], K[1], FK[0], FK[1], b, e, N);
  for(k=0; k<N; k++){
    KrotG[0][k]=K[0][k];
    KrotG[1][k]=K[1][k];
  }
  fft(KrotG[0], KrotF[0], N);
  fft(KrotG[1], KrotF[1], N);
  fourierrot(KrotF[0], KrotF[0], om, N);
  fourierrot(KrotF[1], KrotF[1], om, N);
  ifft(KrotF[0], KrotG[0], N);
  ifft(KrotF[1], KrotG[1], N);
  printf("\nTK1 %.2le \t %.2le\n", l1tail(KrotF[0], N), l1tail(KrotF[1], N));
  printf("TKI %.2le \t %.2le\n",  linftail(KrotF[0],N),  linftail(KrotF[1],N));
  rest(FK[0], KrotG[0], err[0], N);
  rest(FK[1], KrotG[1], err[1], N);
  freev(KrotG);
  freev(KrotF);
  freev(FK);
  {
    double err0, err1;
    err0= supnorm(err[0], N);
    err1= supnorm(err[1], N);
    if(err0>err1)
      return err0;
    else
      return err1;
  }
}

void difmatrix(complex *K[2], complex *dif[2][2], double b, int N){
	/* Evaluates the differential matrix of the Standard Map over K */
  int k;
  for(k=0; k<N; k++){
    dif[0][0][k]=1-b*ccos(2*PI*K[0][k]);
    dif[0][1][k]=1;
    dif[1][1][k]=1;
    dif[1][0][k]=-b*ccos(2*PI*K[0][k]);
  }
}

void dft(complex *grid, complex *coef, int N){
  int j, k;
  long double complex sum;
  for(k=0; k<N; k++){
    sum= 0.;
    for(j=0; j<N; j++){
      sum+= ((long double complex) grid[j])*cexpl(-((long double complex) 2.0l*PI*I*k*j)/N);
    }
    coef[k]= (complex) (sum/N);
  }
}

void idft(complex *coef, complex *grid, int N){
  int j, k;
  long double complex sum;
  for(k=0; k<N; k++){
   	sum= 0;
   	for(j=N/2; j<N; j++){
   		sum+= ((long double complex) coef[j])*cexpl(((long complex)2.0l*PI*I*k*j)/N);
   		sum+= ((long double complex) coef[N-1-j])*cexpl(((long complex) 2.0l*PI*I*k*(N-1-j))/N);
    }
    grid[k]= (complex) sum;
  }
}

void separate (complex *a, int n){
	/* Copies all even elements to lower-half of a[]
	and all odd elements to upper-half of a[] */
  complex b[n/2];
  int i;
  for(i=0; i<n/2; i++)   
    b[i]=a[i*2+1];
  for(i=0; i<n/2; i++)    
    a[i]=a[i*2];
  for(i=0; i<n/2; i++)    
    a[i+n/2]=b[i];          
}

void _fft(complex *X, int N){
	int i, k;
	complex e, o, w;
  if(N<2){
  }else{
    separate(X,N);      
    _fft(X, N/2); 
    _fft(X+N/2, N/2);  
    for(k=0; k<N/2; k++){
      e=X[k];   
      o=X[k+N/2];   
      w=cexp(-2.*PI*I*k/N);
      X[k]=e/2.+w*o/2.;
      X[k+N/2] =e/2.-w*o/2.;
    }
  }
}

void fft(complex *grid, complex *coef, int N){
  int k;
  for (k=0; k<N; k++){
  	coef[k]=grid[k];
  }
  _fft(coef, N);
}

void _ifft(complex *X, int N){
	int i, k;
	complex e, o, w;
  if(N<2){
  }else{
    separate(X,N);      
    _ifft(X, N/2);   
    _ifft(X+N/2, N/2);   
    for(k=0; k<N/2; k++){
      e=X[k];   
      o=X[k+N/2];   
      w=cexp(2.*PI*I*k/N);
      X[k]=e+w*o;
      X[k+N/2]=e-w*o;
    }
  }
}

void ifft(complex *coef, complex *grid, int N){
  int k;
  for(k=0; k<N; k++){
  	grid[k]=coef[k];
  }
  _ifft(grid, N);
}

int main(){
	
  int N= 1024, NMAX= 16384;
  int i, maxiter= 6, j, k, r=0;
  double om=(-1+sqrt(5))/2, b= 1.3, e, e0, eok, step; 
  complex *K[2], *K0[2], *P0[2][2], *KF[2], *deltaK[2], *etaG[2], *etaF[2], *P[2][2], *Prot[2][2], *PinvG[2][2], *PinvF[2][2], *ProtinvG[2][2], *ProtinvF[2][2], *dif[2][2], *invaerr[2], *reduerrG[2][2], *reduerrF[2][2], *inverserr[2][2], 
  *auxG[2][2], *auxF[2][2], *QF[2][2], *QG[2][2], *xiF[2], *xiG[2], v0[2], v1[2];
  double lam0, lam1, delta0, delta1, error, errorredu, totalerror, tolerror= 1.e-9, tolerrorredu= 1.e-7, tail, ang0, ang1, mod, dmin, dmax, d;
  char duplicate;
  FILE *globalfile;
  char generalfile[100];

  globalfile= fopen("info.txt", "w");
  if (!globalfile) {
   	puts("File Error");
  }
	
  allocv(etaG, N);
  allocv(etaF, N);
  allocv(invaerr, N);
  allocv(xiG, N);
  allocv(xiF, N);
  allocv(K, N);
  allocv(K0, N);
  allocv(KF, N);
  allocv(deltaK, N);
  allocm(P, N);
  allocm(ProtinvG, N);
  allocm(Prot, N);
  allocm(ProtinvF, N);
  allocm(PinvG, N);
  allocm(PinvF, N);
  allocm(dif, N);
  allocm(reduerrG, N);
  allocm(reduerrF, N);
  allocm(QG, N);
  allocm(QF, N);
  allocm(inverserr, N);
  allocm(auxG, N);
  allocm(P0, N);
  allocm(auxF, N);
  	
  /* Start off with e=0, and hyperbolic fixed point (1/2, 0), filling P with eigenvectors and Lambda with its eigenvalues */
  lam0=(2+b+sqrt(b*(b+4)))/2;
  lam1=(2+b-sqrt(b*(b+4)))/2;

  for(k=0; k<N; k++){
    K0[0][k]=1/2.;
    K0[1][k]=0;
    P0[0][0][k]=sqrt(2)/sqrt(b*b+2*b-b*sqrt(b*(b+4))+2);
    P0[1][0][k]=(-b+sqrt(b*(b+4)))/sqrt(2*b*b+4*b-2*b*sqrt(b*(b+4))+4);
    P0[0][1][k]=(b-sqrt(b*(b+4)))/sqrt(2*b*b+4*b-2*b*sqrt(b*(b+4))+4);
    P0[1][1][k]=sqrt(2)/sqrt(b*b+2*b-b*sqrt(b*(b+4))+2);
  }
  printf("\nK (%02le \t %02le)", creal(K[0][0]), creal(K[1][0])); fflush(stdout);

  e0=0;
  eok= 0;
  
  step=1e-1;
  duplicate= 0;
  do{   
    e=e0+step;

    for(k=0; k<N; k++){
      K[0][k]=K0[0][k];
      K[1][k]=K0[1][k];
      P[0][0][k]=P0[0][0][k];
      P[0][1][k]=P0[0][1][k];
      P[1][0][k]=P0[1][0][k];
      P[1][1][k]=P0[1][1][k];
    }	
    	
		printf("\n\nEPS= %lf\n", e);
    i= 0;
    do{
    	/* This loop applies the Newton method */
      printf("\n\nSTEP %i\n", i+1);
      i++;
      
      /* Step 1: Calculate Invariance Error */
      error=inverr(K, invaerr, b, e, om, N);
      printf("\nInvariance error (%02le)", error);
      if(error>100.){
				i=maxiter;
				break;
      }
      
      inverse(P, PinvG, N);
      matrixgf(PinvG, PinvF, N);
      fourierrot(PinvF[0][0], ProtinvF[0][0], om, N);
      fourierrot(PinvF[0][1], ProtinvF[0][1], om, N);
      fourierrot(PinvF[1][0], ProtinvF[1][0], om, N);
      fourierrot(PinvF[1][1], ProtinvF[1][1], om, N);
      matrixfg(ProtinvF, ProtinvG, N);
      matrixvectmult(ProtinvG, invaerr, etaG, N);

      if(error<tolerror) goto step3;

    	/* Step 2: K correction and reducibility error */
      esc(etaG[0], etaG[0],-1, N);
      esc(etaG[1], etaG[1],-1, N);
      fft(etaG[0], etaF[0], N);
      fft(etaG[1], etaF[1], N);
      cohom1(etaF[0], xiF[0], lam0, 1, om, N);
      cohom1(etaF[1], xiF[1], lam1, 1, om, N);
      ifft(xiF[0], xiG[0], N);
      ifft(xiF[1], xiG[1], N);
          
      matrixvectmult(P, xiG, deltaK, N);
      sum(K[0], deltaK[0], K[0], N);
      sum(K[1], deltaK[1], K[1], N);
      printf("\nNew K(0) = (%02le \t %02le)", creal(K[0][0]), creal(K[1][0])); fflush(stdout);

    	/* Step 3: Calculate Q and deltas */
    	step3:
      difmatrix(K, dif, b, N);
      reducerr(ProtinvG, dif, P, lam0, lam1, reduerrG, N);
      errorredu=infnorm(reduerrG, N);
      printf("\nReducibility Error Norm (%02le)", errorredu);
      printf("\nReducibility Error (%02le %02le)", supnorm(reduerrG[0][0], N), supnorm(reduerrG[0][1], N));
      printf("\n (%02le %02le)\n", supnorm(reduerrG[1][0], N), supnorm(reduerrG[1][1], N)); 
      if(errorredu>100.){
				i= maxiter;
				break;
      }
    	if(errorredu<tolerrorredu) goto step5;
      
      /* Step 4: Correction of P and lambda */
      matrixgf(reduerrG, reduerrF, N);
      cohom1(reduerrF[0][1], QF[0][1], -lam0, -lam1, om, N);
      cohom1(reduerrF[1][0], QF[1][0], -lam1, -lam0, om, N);
      delta0=reduerrF[0][0][0];
      delta1=reduerrF[1][1][0];
      cohom2(reduerrF[0][0], QF[0][0], -lam0, om, N);
      cohom2(reduerrF[1][1], QF[1][1], -lam1, om, N);
      
      matrixfg(QF, QG, N);
      matrixmult(P, QG, auxG, N);
      matrixsum(P, auxG, N);
      lam0+=delta0;
      lam1+=delta1;

    	step5:
      /* Checks Fourier tails */
      fft(K[0], KF[0], N);
      fft(K[1], KF[1], N);
      matrixgf(P, PinvF, N);
      
      {		
				double t0, t1;
				t0=linftail(KF[0], N); t1=linftail(KF[1], N);
				tail= t0>t1 ? t0 : t1;
      }
      
      if(isnan(tail) || tail>tolerror) i=maxiter ;
      		
    } while((error>tolerror || errorredu>tolerrorredu) && i<maxiter);

    if( i==maxiter && ((error>tolerror || errorredu>tolerrorredu))){
      if(duplicate){
				step*=0.1;
				duplicate=0;
				continue;
      }
      duplicate=1;
      N*=2;
      if(N>NMAX) break;
      printf("Duplication to %i\n", N);
      
      reallocv(etaG, N);
      reallocv(etaF, N);
      reallocv(invaerr, N);
      reallocv(xiG, N);
      reallocv(xiF, N);
      reallocv(K, N);
      reallocv(K0, N);
      reallocv(KF, N);
      reallocv(deltaK, N);
      reallocm(P, N);
      reallocm(P0, N);
      reallocm(ProtinvG, N);
      reallocm(Prot, N);
      reallocm(ProtinvF, N);
      reallocm(PinvG, N);
      reallocm(PinvF, N);
      reallocm(dif, N);
      reallocm(reduerrG, N);
      reallocm(reduerrF, N);
      reallocm(QG, N);
      reallocm(QF, N);
      reallocm(inverserr, N);
      reallocm(auxG, N);
      reallocm(auxF, N);

      if(i==maxiter && ((error>tolerror || errorredu>tolerrorredu))){
				printf("Reduction of step\n");
				step*= 1.0e-1;
				fft(K0[0], KF[0], N/2);
				fft(K0[1], KF[1], N/2);
				matrixgf(P0, PinvF, N/2);	
      }
 	
      for(k=N/4; k<N/2; k++){
				KF[0][k+N/2]=KF[0][k];
				KF[1][k+N/2]=KF[1][k];
				PinvF[0][0][k+N/2]=PinvF[0][0][k];
				PinvF[0][1][k+N/2]=PinvF[0][1][k];
				PinvF[1][0][k+N/2]=PinvF[1][0][k];
				PinvF[1][1][k+N/2]=PinvF[1][1][k];
      }
      	
      for(k=N/4; k<3*N/4; k++){
				KF[0][k]=0;
				KF[1][k]=0;
				PinvF[0][0][k]=0;
				PinvF[0][1][k]=0;
				PinvF[1][0][k]=0;
				PinvF[1][1][k]=0;
      }

      {
				printf("TK1 %.2le \t %.2le\n",  l1tail(KF[0],N),  l1tail(KF[1],N));
				printf("TKI %.2le \t %.2le\n",  linftail(KF[0],N),  linftail(KF[1],N));
	
				ifft(KF[0], deltaK[0], N);
				ifft(KF[1], deltaK[1], N);

				fft(deltaK[0], KF[0], N);
				fft(deltaK[1], KF[1], N);

				printf("TK1 %.2le \t %.2le\n",  l1tail(KF[0],N),  l1tail(KF[1],N));
				printf("TKI %.2le \t %.2le\n",  linftail(KF[0],N),  linftail(KF[1],N));
      }

      ifft(KF[0], K0[0], N);
      ifft(KF[1], K0[1], N);
      matrixfg(PinvF, P0, N);
    }else{
      duplicate= 0;
      if(i<=3 && e0!=0) step*=10.;
      e0=e;
      for(k=0; k<N; k++){
				K0[0][k]=K[0][k];
				K0[1][k]=K[1][k];
				P0[0][0][k]=P[0][0][k];
				P0[0][1][k]=P[0][1][k];
				P0[1][0][k]=P[1][0][k];
				P0[1][1][k]=P[1][1][k];
      }

      {
				FILE *file;
				char myfile[100];
				sprintf(myfile, "K%.6lf.txt", e0);
				printf("%s\n", myfile);
				file= fopen(myfile, "w");
				if(!file){
	  			puts("File Error");
				}else{
	  			fprintf(globalfile,"%lf %le %le ", e0, lam0, lam1);	  
	  			dmin= sqrt(2.);
	  			dmax= 0;

	  			for(k=0; k<N; k++){	    
	    			ang0=atan2(creal(P0[0][0][k]), creal(P0[1][0][k]));
	    			if(ang0<0) ang0+=PI;
	    			ang1=atan2(creal(P0[0][1][k]), creal(P0[1][1][k]));
	    			if(ang1<0) ang1+=PI;
	    
	    			fprintf(file, "%.8lf % .8lf % .8lf % .8lf % .8lf\n", (double)k/N, creal(K0[0][k]), creal(K0[1][k]), ang0, ang1);	 
 
	    			v0[0]=creal(P[0][0][k]); v0[1]=creal(P[1][0][k]);
	    			v1[0]=creal(P[0][1][k]); v1[1]=creal(P[1][1][k]);
	    			mod=sqrt(v0[0]*v0[0]+v0[1]*v0[1]);
	    			v0[0]/=mod, v0[1]/=mod;
	    			mod=sqrt(v1[0]*v1[0]+v1[1]*v1[1]);
	    			v1[0]/=mod, v1[1]/=mod;

	    			double d1, d2;
	    			d1=hypot(v0[0]-v1[0],v0[1]-v1[1]);
	    			d2=hypot(v0[0]+v1[0],v0[1]+v1[1]);
	    			d=d1 < d2 ? d1 : d2;
	    
  					if(d<dmin) dmin=d;
						if(d>dmax) dmax=d;
	  			}
	  			fprintf(globalfile, " %le %le\n", dmin, dmax);
	  			fclose(file);
				}	
			}   
		}
   
  } while(step>1e-4);

	fclose(globalfile);
  printf("\n\nFINAL EPS OK= %lf\n", e0);
  printf("\n\nFINAL EPS KK= %lf\n", e);
  printf("FINAL N=   %i\n", N);

  freev(etaG);
  freev(etaF);
  freev(invaerr);
  freev(xiG);
  freev(xiF);
  freev(K);
  freev(KF);
  freev(deltaK);
  freem(P);
  freem(ProtinvG);
  freem(Prot);
  freem(ProtinvF);
  freem(PinvG);
  freem(PinvF);
  freem(dif);
  freem(reduerrG);
  freem(reduerrF);
  freem(QG);
  freem(QF);
  freem(inverserr);
  freem(auxG);
  freem(auxF);

  return 0;
}	
